#######################################################################################################################
#######################################################################################################################
####### Continuous Time Meta Analysis (CoTiMA) - R script Version 1.1 (cf. Dormann, Guthier, & Cortina, 2019) #########
#######################################################################################################################
#######################################################################################################################

# contact: cdormann@uni-mainz.de

# This R script performs a CoTiMA based on correlation matrices of primary studies.
# Necessary Steps:
#   1. Download and install R (https://cran.r-project.org). 
#      You may also want to install RStudio (https://www.rstudio.com), which is a desktop program facilitating using R.
#   2. Open this file in Rstudio.
#   3. Change the 3 "Presets that SHOULD be adapted" in the Subsection "Presets" below:
#      3a) Enter the working directory where this file is stored. Any output will be directed to this directory, too.
#      3b) Enter a file name for the results (e.g., "results.txt")
#      3c) Enter a file "prefix" (e.g., "result") that will be used for all saved figures (e.g., "result_CrossXtoY.png")
#   4. In the Subsection "Primary Study Information",  for each primary study, enter (check spelling & capitalization) ...
#      4a) ... the time delta_t(s) used in the primary studies  (e.g., delta_t1 <- 3 or delta_t2 <- c(1, 2, 5) for three ...
#          ... different delta_ts). Note that "t1", "t2" refer to the study numbers. 
#      4b) ... the sample sizes (sampleSize1 <- 668, sampleSize2 <- 22 etc.).
#      4c) ... the correlation matrix of the primary study variables X and Y at all time points (X0, Y0, X1, Y1, etc.)
#   5. Mark the entire script and select "Run" 

#######################################################################################################################
################################################### Presets ###########################################################
#######################################################################################################################

### Presets that SHOULD be adapted

# Working directory to save results, & prefix for figure file names. Use "/" instead of "\".
workingDirectory <- "/Users/.../"  				
resultsFileName <- "CoTiMA SatPerf.txt"
figureFilePrefix <- "CoTiMA SatPerf"

### Primary Study Information
delta_t1 <-6; 
sampleSize1 <- 222
empcov1 <- matrix(c(
  1,	  0.11,	0.64,	0.14,
  0.11, 1,   	0.2,	0.69,
  0.64,	0.2,	1,	  0.21,
  0.14,	0.69,	0.21,	1), nrow=4, ncol=4)
delta_t2 <-18
sampleSize2 <- 64
empcov2 <- matrix(c(
  1,	0.15,	0.53,	0.17,	
  0.15,	1,	0.19,	0.57,
  0.53,	0.19,	1,	0.21,
  0.17,	0.57,	0.21,	1), nrow=4, ncol=4)
chol(empcov2)
delta_t3 <-12
sampleSize3 <- 412
empcov3 <- matrix(c(
  1,	0.26,	0.66,	0.05,	
  0.26,	1,	0.66,	0.21,
  0.66,	0.66,	1,	0.17,
  0.05,	0.21,	0.17,	1), nrow=4, ncol=4)
delta_t4 <-18
sampleSize4 <- 526
empcov4 <- matrix(c(
  1,	0.04,	0.61,	-0.02,
  0.04,	1,	0.06,	0.53,
  0.61,	0.06,	1,	0.06,
  -0.02,	0.53,	0.06,	1), nrow=4, ncol=4) 
delta_t5 <-3.5
sampleSize5 <- 300
empcov5 <- matrix(c(
  1,	0.06,	0.56,	0.14,
  0.06,	1,	-0.02,	0.23,
  0.56,	-0.02,	1,	0.17,
  0.14,	0.23,	0.17,	1), nrow=4, ncol=4) 
delta_t6 <- 11
sampleSize6 <- 59
empcov6 <- matrix(c(
  1,	-0.03,	0.45,	-0.08,	
  -0.03,	1,	-0.06,	0.5,
  0.45,	-0.06,	1,	0.15,
  -0.08,	0.5,	0.15,	1), nrow=4, ncol=4)
delta_t7 <-12
sampleSize7 <- 35
empcov7 <- matrix(c(
  1,	0.2,	0.68,	0.21,	
  0.2,	1,	0.24,	0.49,
  0.68,	0.24,	1,	0.21,
  0.21,	0.49,	0.21,	1), nrow=4, ncol=4)
delta_t8 <-3
sampleSize8 <- 128
empcov8 <- matrix(c(
  1,	0.09,	0.62,	0.09,
  0.09,	1,	0.09,	0.65,
  0.62,	0.09,	1,	0.05,
  0.09,	0.65,	0.05,	1), nrow=4, ncol=4)
delta_t9 <-12
sampleSize9 <- 200
empcov9 <- matrix(c(
  1,	0.19,	0.48,	0.11,
  0.19,	1,	0.08,	0.64,
  0.48,	0.08,	1,	0.08,
  0.11,	0.64,	0.08,	1),  nrow=4, ncol=4)
delta_t10 <-2
sampleSize10 <- 80
empcov10 <- matrix(c(
  1,	0.09,	0.73,	0.18,
  0.09,	1,	0.24,	0.44,
  0.73,	0.24,	1,	0.15,
  0.18,	0.44,	0.15,	1), nrow=4, ncol=4)


####################################### Presets that COULD be adapted #################################################
############ (adjust your expectations: If all set "TRUE" this example script clould take up to hours) ################

nlatents <- 2                     # select 1 for 1 variable case, 2 for 2 variable case

retryattempts <- 50               # number of attempts to fit a model (similar to "number of iterations", e.g., in ...
#                                 # ... factor analysis )

refits <- 20                      # how many times all retryattempts should be re-done. We recommend 10-20 if the ...
#                                 # .... script runs smoothly. Start with 1.

digits <- 3                       # number of digits displayed in results file

confidenceIntervals <- TRUE       # Computing confidence intervals is time consuming. We recommend to set to ...
#                                 # ... TRUE after CoTiMA runs without problems.

testDRIFTallModel <- TRUE         # test a homogeneity model with all drift effects invariant across primary ...
#                                 # ... studies (= recommended CoTiMA)
testDRIFTSingleModel <- FALSE     # test a series of homogeneity models with single drift effects ...
#                                 # ... invariant (across primary studies).

testDRIFTCrossModel <- TRUE       # test a homogeneity model with both cross effects to be identical and invariant.
testDRIFTAutoModel <- FALSE       # test a homogeneity model with both auto effects to be identical and invariant.

fixedAndRandomEffects <- TRUE     # use estimated cross and auto effects of primary studies to perform fixed and ...
#                                 # ... random effects analysis (Borenstein et al., 2009)

plotCrossEffects <- TRUE          # plot discrete time cross-lagged effects of CoTiMA results (recommended).
plotAutoEffects <- TRUE           # plot discrete time autoregressive effects of CoTiMA results.

NPSOL <- TRUE                     # in fitting the model, various so-called optimizers could be applied. NPSOL is such ...
#                                 # ... an optimizer but usually requires manual installation.  If set to "TRUE" ...
#                                 # ... this is done (by replacing OpenMx from CRAN by OpenMx from ssri.psu.edu)

testHeterogeneityModel <- FALSE   # a heterogeneity model could be tested, but fitting may take long (!) time and ...
#                                 # ... results are redundant (it is identical to all studies fitted separately, which ...
#                                 # ... is always done); (not recommended, except for possible diagnosis of error ...
#                                 # ... in fitting homogeneity model). Included for the sake of completeness only.



####################################### Do not edit the lines below ###################################################
# Set the working directory & run the 2nd CoTiMA script that performs all analyses and saves all results and figures
setwd(workingDirectory)  	
print("When using CoTiMA/R for the first time, it may take up to 15min to install required packages.")
print("You may not see too much during this period. When asked if you want to restart R, respond NO.")
source(paste0(workingDirectory, "Online Supplement S-2 CoTiMA V1.4 noedit.R"))
